gem 'test-unit'
gem 'test-unit', '<= 2.1.1' if RUBY_VERSION == '1.9.1'
require 'test/unit'

=begin
The original rail.exe is a 16bit version and works not
with Windows 64bit.

The compilation is no problem:

1. Get rail source from http://ctan.org/tex-archive/support/rail
2. You need devkit-tools (make, yacc...)
  To use the ruby devkit:
    PATH=%PATH%;C:\bin\devkit\mingw\bin;C:\bin\devkit\bin
    make
=end

#~ require 'assert_equal_filecontent'
if $0 == __FILE__
  $:.unshift('../lib')
else
  #Change the directory. Relevant, if called from rakefile_rake4latex.rb
  Dir.chdir(File.dirname(__FILE__))
end
require 'rake4latex'
include Rake4LaTeX
require_relative('rake4latex_testhelper')

class Test_rail < Test::Unit::TestCase
  def test_rail_available
    found = false
    ENV['Path'].split(/;/).each{|path|
      next if path.empty?
      Dir.chdir(path){
        found = true if File.exist?( 'rail.exe' )
      } rescue Errno::ENOENT
      break if found
    }
    assert_true(found, 'Rail.exe not found in search path')
  end
  def test_correct_programm
    stdin, stdout, stderr, wait_thr = Open3.popen3('rail.exe xxx')
    assert_true(wait_thr.value.exited?) #Process::Status
    assert_false(wait_thr.value.success?) #Process::Status
    assert_equal(1, wait_thr.value.exitstatus ) #Process::Status
    #~ assert_equal([%{This is Rail, Version 1.0 #0\n}], stdout.readlines) #Win32
    assert_equal([%{This is Rail, Version 1.2 #1\n}], stdout.readlines)
    assert_equal([%{xxx.rai: No such file or directory\n}], stderr.readlines)
  end
end

class Test_rail_ok < Test::Unit::TestCase
  def self.startup
    @@testrai_file = '_tmp_.rai'
    @@delete_at_teardown = [@@testrai_file]
    File.open(@@testrai_file, 'w'){|f|
      f << "\\rail@i{1}{ block1 : ( lbrace | 'begin' ) ( ( '|' ( variable + ',' ) '|' ) | [Keine Blockvariablen] ) comm ( rbrace | 'end' ) ; }"
    }
  end
  def self.shutdown
    @@delete_at_teardown.each{|filename|
      (File.delete(filename) if File.exist?(filename)) rescue Errno::EACCES
    }
  end
  def setup
    @rail = Rake4LaTeX::Rail.new(@@testrai_file)
  end

  def test_rail_cmd
    assert_equal("rail.exe < #{@@testrai_file}", @rail.cmd)
  end
  def test_rail_cmd_execution
    expected = <<'expect'
\rail@i {1}{ block1 : ( lbrace | 'begin' ) ( ( '|' ( variable + ',' ) '|' ) | [Keine Blockvariablen] ) comm ( rbrace | 'end' ) ; }
\rail@o {1}{
\rail@begin{3}{block1}
\rail@bar
\rail@nont{lbrace}[]
\rail@nextbar{1}
\rail@term{begin}[]
\rail@endbar
\rail@bar
\rail@term{|}[]
\rail@plus
\rail@nont{variable}[]
\rail@nextplus{1}
\rail@cterm{,}[]
\rail@endplus
\rail@term{|}[]
\rail@nextbar{2}
\rail@annote[Keine Blockvariablen]
\rail@endbar
\rail@nont{comm}[]
\rail@bar
\rail@nont{rbrace}[]
\rail@nextbar{1}
\rail@term{end}[]
\rail@endbar
\rail@end
}
expect

    assert_true( @rail.execute )
    assert_equal(expected.strip,@rail.stdout)
    assert_equal('', @rail.stderr)
  end
end #Test_Rail_ok



class Test_rail_error < Test::Unit::TestCase
  def self.startup
    @@testrai_file = '_tmp_.rai'
    @@delete_at_teardown = [@@testrai_file]
    File.open(@@testrai_file, 'w'){|f|
      #rail-definition with error
      f << "\rail@i{1}{ block1 : ( "
    }
  end
  def self.shutdown
    @@delete_at_teardown.each{|filename|
      (File.delete(filename) if File.exist?(filename)) rescue Errno::EACCES
    }
  end
  def setup
    @rail = Rake4LaTeX::Rail.new(@@testrai_file)
  end
  def test_rail_cmd
    assert_equal("rail.exe < #{@@testrai_file}", @rail.cmd)
  end
  def test_rail_cmd_execution
    assert_false( @rail.execute )
    assert_equal("",@rail.stdout.strip)
    assert_equal(%{stdin, line 1: syntax error\n}, @rail.stderr)
  end
end #Test_Rail_error


class Test_rail_testcases < Test::Unit::TestCase
  include Rake4latex_testhelper #get define_testcase

  def test_testdirectories()
    assert_true(Dir.exist?('rail'))
    assert_true(Dir.exist?('rail_error'))
  end
      
  def test_rail
    define_testcase( 'rail')
  end

  def test_rail_error
    define_testcase( 'rail_error')
  end
end
