#
#Unit tests without TeX-Calls (see unittest_rake4latex_testcases.rb for thos tests.)
#

gem 'test-unit'
require 'test/unit'
#~ require 'assert_equal_filecontent'
if $0 == __FILE__
  $:.unshift('../lib')
else
  #Change the directory. Relevant, if called from rakefile_rake4latex.rb
  Dir.chdir(File.dirname(__FILE__))
end
require 'rake4latex'
include Rake4LaTeX

class Test_Basefile < Test::Unit::TestCase
  def test_initialize()
    assert_raise(NoMethodError){ Basefile.new() } #private method
    assert_raise(ArgumentError){ Basefile.set() }
    
    assert_equal(nil, Basefile['test_basefile.tex'])
    assert_nothing_raised(ArgumentError){ Basefile.set( 'test_basefile.tex') }
    #Test different variations with different extensions.
    assert_equal(Basefile['test_basefile.tex'], Basefile.set( 'test_basefile.tex'))
    assert_equal(Basefile['test_basefile.tex'], Basefile.set( 'test_basefile'))
    assert_equal(Basefile['test_basefile'], Basefile.set( 'test_basefile.tex'))
    assert_equal(Basefile['test_basefile.pdf'], Basefile.set( 'test_basefile.tex'))

    #Undefined key.
    assert_raise(ArgumentError){ Basefile.set( 'test_basefile.tex', :xx => 1) }
    assert_nothing_raised(ArgumentError){ Basefile.set( 'test_basefile.tex', :loglevel => 1) }

  end #test_initialize
end
class Test_LaTeXRunner < Test::Unit::TestCase
  def test_initialize()
    #~ texrunner = LaTeXRunner.new( :main_file => 'testdocument.tex' )

    #Avoid messages
    LaTeXRunner::DEFAULT_SETTINGS[:loglevel] = Log4r::FATAL

    #exception Unknown option
    assert_raise(ArgumentError){ LaTeXRunner.new( :main_file => 'testdocument.tex', :unknown_option => 'a' ) }

    #exception main file not found/missing
    assert_raise(ArgumentError){ LaTeXRunner.new( ) }
    assert_raise(LaTeXRunner::MainfileError){ LaTeXRunner.new( :main_file => nil) }
    assert_raise(LaTeXRunner::MainfileError){ LaTeXRunner.new( :main_file => 'unknown_document.tex') }

    #exception maxrun
    assert_raise(ArgumentError){ LaTeXRunner.new( :main_file => 'testdocument.tex', :maxruns => 'a' ) }
    assert_raise(ArgumentError){ LaTeXRunner.new( :main_file => 'testdocument.tex', :maxruns => 1.2 ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :maxruns =>  LaTeXRunner::Infinity ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :maxruns =>  1.0/0 ) }

    #exception program
    assert_raise(ArgumentError){ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => :unknown ) }
    assert_raise(ArgumentError){ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => 'latex' ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => :latex ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => :pdflatex ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => :lualatex ) }
    assert_nothing_raised{ LaTeXRunner.new( :main_file => 'testdocument.tex', :program => :xelatex ) }

  end
end

class Test_LaTeXRunner_Programms < Test::Unit::TestCase
  
  def test_generell()
    assert_instance_of(Hash, Rake4LaTeX::Programms)
    Rake4LaTeX::Programms.each{|tool, options |
      assert_instance_of(Hash, options )
      #Check valid keys. comment is ok, but not needed
      options.delete( 'comment')
      assert_equal(%w{cmd parameters}, options.keys.sort)
      
      assert_instance_of(String, options['cmd'] )
      
      assert_instance_of(Array, options['parameters'] )
      options['parameters'].each{|parameter|
        assert_instance_of(Hash, parameter )
      }
    }
    
  end #test_generell
  #
  #Test all LaTeX-variants
  def test_latex_single()
    %w{latex pdflatex lualatex xelatex}.each{|latex|
      tool = Rake4LaTeX::Programms[latex]
      assert_instance_of(Hash, tool)
      assert_equal(latex, tool['cmd'])
      assert_equal("#{latex} -interaction=batchmode test.tex",
            Rake4LaTeX.build_cmd(latex, :filename => 'test.tex' )
          )
      assert_equal("#{latex} -interaction=batchmode -output-directory=out test.tex",
            Rake4LaTeX.build_cmd(latex, :filename => 'test.tex', :output_dir => 'out' )
          )
    }
  end
  #Test all LaTeX-variants
  def test_bibtex()
    tool = Rake4LaTeX::Programms['bibtex']
    assert_instance_of(Hash, tool)
    assert_equal('bibtex', tool['cmd'])
    assert_equal("bibtex test.aux",
          Rake4LaTeX.build_cmd('bibtex', :source => 'test.aux' )
        )
  end
  def test_makeindex()
    tool = Rake4LaTeX::Programms['makeindex']
    assert_instance_of(Hash, tool)
    assert_equal('makeindex', tool['cmd'])
    assert_equal("makeindex test.idx",
            Rake4LaTeX.build_cmd( 'makeindex', :file_in => 'test.idx' )
        )
  end
  #Test all LaTeX-variants
  def test_latex_with_postprocessors()
    %w{dvipdfm dvips ps2pdf}.each{|tool_id|
      tool = Rake4LaTeX::Programms[tool_id]
      assert_instance_of(Hash, tool)
      assert_equal(tool_id, tool['cmd'])
      assert_equal("#{tool_id} test.tex",
            Rake4LaTeX.build_cmd(tool_id, :filename => 'test.tex' )
          )
    }
  end
end #Test_LaTeXRunner_Programms

class Test_analyse < Test::Unit::TestCase
  def test_initialize()
    
    assert_raise(ArgumentError){ TeXfile.new( 'not_existing_document.tex' ) }
    
  end
  
  def test_includes()
    Dir.chdir('includes'){
      testfile = TeXfile.new('testdocument.tex')
      assert_equal( ["testdocument.tex",
                             ["testincludes/testinclude1.tex", ["testincludes/testinclude2.tex"]],
                             ["testincludes/testinclude1.tex", ["testincludes/testinclude2.tex"]]], 
                          testfile.tree )
      assert_equal( ["testincludes/testinclude1.tex", "testincludes/testinclude1.tex"], testfile.includes )
      assert_equal( ["testincludes/testinclude1.tex"], testfile.includes(:uniq) )
      assert_equal( ["testincludes/testinclude1.tex", "testincludes/testinclude2.tex" ], testfile.includes(:recursive) )
      #~ assert_equal( ["testdocument.tex", "testincludes/testinclude1.tex", "testincludes/testinclude2.tex" ], testfile.includes(:recursive) )
    }
  end
  def test_bibtex()
    Dir.chdir('bibtex'){
      testfile = TeXfile.new('testdocument.tex')
      assert_equal( ['testdocument.tex'], testfile.tree )
      assert_equal( ['./testdocument.bib'], testfile.bibliography )
      assert_equal( {}, testfile.gloss )
    }
    
    testfile = Rake4LaTeX::TeXfile.new(:dummy)
    testfile.analyse(<<'xx'
\bibliography{dd}
\bibliography{testdocument}
xx
    )
    assert_equal( ['dd.bib', 'testdocument.bib'], testfile.bibliography )

    testfile = Rake4LaTeX::TeXfile.new(:dummy)
    testfile.analyse(<<'xx'
\bibliography{dd,testdocument}
xx
    )
    assert_equal( ['dd.bib', 'testdocument.bib'], testfile.bibliography )

  end
  def test_gloss()
    Dir.chdir('gloss'){
      testfile = TeXfile.new('testdocument.tex')
      assert_equal( ['testdocument.tex'], testfile.tree )
      assert_equal( [], testfile.bibliography )
      assert_equal( {"gls"=>["./test_gloss.bib"], "[animals]"=>["./test_gloss.bib"]}, testfile.gloss )
    }
  end

end
