gem 'more_unit_test', '>0.0.2'
gem 'test-unit'
require 'test/unit'
#~ require 'assert_equal_filecontent'
if $0 == __FILE__
  $:.unshift('../lib')
else
  #Change the directory. Relevant, if called from rakefile_rake4latex.rb
  Dir.chdir(File.dirname(__FILE__))
end
require 'rake4latex'
include Rake4LaTeX
require_relative('rake4latex_testhelper')

class Test_Splitindex < Test::Unit::TestCase
  def self.startup
    @@testidx_file = '_tmp_splitindex.idx'
    @@delete_at_teardown = []
    @@delete_at_teardown << @@testidx_file
    File.open(@@testidx_file, 'w'){|f|
      f << <<idx
\indexentry[fru]{apple|hyperpage}{1}
\indexentry[fru]{orange|hyperpage}{1}
\indexentry{fruits|hyperpage}{1}
\indexentry[veg]{tomato|hyperpage}{1}
\indexentry{vegetables|hyperpage}{1}
\indexentry[ani]{cat|hyperpage}{1}
\indexentry[idx]{animals|hyperpage}{1}      
idx
    }
  end
  def self.shutdown
    @@delete_at_teardown.each{|filename|
      File.delete(filename) if File.exist?(filename)
    }
  end
  def setup
    @log = Log4r::Logger.new('log')
  end
  #~ def test_interface
    #no exception thrown, only logger error
    #~ assert_raise(ArgumentError, 'idx-file not found'){
        #~ Rake4LaTeX::Splitindex.new(@@testidx_file * 2, @log)
    #~ }
  #~ end
  
  def test_splitindex
    sidx = nil
    assert_nothing_raised{ sidx = Rake4LaTeX::Splitindex.new(@@testidx_file, @log)}
    assert_equal(["fru", "idx", "veg", "ani"], sidx.indices)
    assert_equal(sidx.indices, sidx.targets.keys)
    
    #test ani
    assert_equal(["\\indexentry{cat|hyperpage}{1}"], sidx.targets['ani'])

    #test veg
    assert_equal(["\\indexentry{tomato|hyperpage}{1}"], sidx.targets['veg'])
    
    #test fru
    assert_equal([
        "\\indexentry{apple|hyperpage}{1}",
        "\\indexentry{orange|hyperpage}{1}",
      ], sidx.targets['fru'])

    #test idx
    assert_equal([
        "\\indexentry{fruits|hyperpage}{1}",
        "\\indexentry{vegetables|hyperpage}{1}",
        "\\indexentry{animals|hyperpage}{1}"
      ], sidx.targets['idx'])
  end
end #Test_Splitindex


class Test_analyse < Test::Unit::TestCase
  
  include Rake4latex_testhelper #get define_testcase

  def test_texfile_splitindex()
    assert_true(Dir.exist?('splitindex'))
    
    Dir.chdir('splitindex'){
      testfile = TeXfile.new('testdocument.tex')
      assert_equal( ['testdocument.tex'], testfile.tree )
      assert_equal( {"veg"=>"[Index of Vegetables]",
                           "ani"=>"[Index of Animals]",
                           "idx"=>"[GeneralIndex]",
                           "fru"=>"[Index of Fruits]"}, 
                          testfile.splitindex, 'indices not found' )
    }
  end
  
  def test_splitindex
    define_testcase( 'splitindex')
  end

end